//==============================================================
//	萔
//==============================================================
float4x4 World;
float4x4 View;
float4x4 Proj;
float4 Diffuse;
float4 Ambient;
float3 Light;

texture MeshTex;
texture ToonTex;

texture BlendTex;
float BlendRate;

float I;

//==============================================================
//	eNX`Tv
//==============================================================
sampler MeshSmp=sampler_state {
	Texture=<MeshTex>;
	MipFilter=NONE;
	MinFilter=NONE;
	MagFilter=NONE;
};

sampler ToonSmp=sampler_state {
	Texture=<ToonTex>;
	MipFilter=NONE;
	MinFilter=NONE;
	MagFilter=NONE;
	AddressU=CLAMP;
};

sampler MeshSmpYIQ=sampler_state {
	Texture=<MeshTex>;
	MipFilter=POINT;
	MinFilter=LINEAR;
	MagFilter=LINEAR;
};

sampler BlendSmp=sampler_state {
	Texture=<BlendTex>;
	MipFilter=LINEAR;
	MinFilter=LINEAR;
	MagFilter=LINEAR;
};

//****************************************************************************
//	x[VbN
//****************************************************************************
//==============================================================
// _VF[_
void BasicVS (
	float4 pos : POSITION,
	float3 normal : NORMAL,
	float2 mesh_uv : TEXCOORD0,
	out float4 Pos : POSITION,
	out float4 Col : COLOR0,
	out float2 MeshUV : TEXCOORD0
) {
	// _WAeNX`W
	Pos=mul(pos, mul(mul(World, View), Proj));
	MeshUV=mesh_uv;

	// _F
	float3 w_normal=normalize(mul(normal, (float3x3)World));
	Col=max(0, dot(w_normal, -Light))+Ambient;
}

//==============================================================
// sNZVF[_
float4 BasicPS(
	float4 Col : COLOR0,
	float2 MeshUV : TEXCOORD0
) : COLOR {
	// eNX`Fƒ_F̍
	return tex2D(MeshSmp, MeshUV)*Col*Diffuse;
}

//==============================================================
// eNjbN
technique Basic {
	pass P0 {
		CullMode=CCW;
		VertexShader=compile vs_1_1 BasicVS();
		PixelShader=compile ps_1_1 BasicPS();
	}
}

//****************************************************************************
//	gD[
//****************************************************************************
//==============================================================
// _VF[_
void ToonPaintVS (
	float4 pos : POSITION,
	float3 normal : NORMAL,
	float2 mesh_uv : TEXCOORD0,
	out float4 Pos : POSITION,
	out float2 MeshUV : TEXCOORD0,
	out float2 ToonUV : TEXCOORD1
) {
	// _WAeNX`W
	Pos=mul(pos, mul(mul(World, View), Proj));
	MeshUV=mesh_uv;
	
	// gD[eNX`W
	float3 w_normal=normalize(mul(normal, (float3x3)World));
	ToonUV.x=max(0, dot(w_normal, -Light))+Ambient;
	ToonUV.y=0.5;
}

//==============================================================
// sNZVF[_
float4 ToonPaintPS(
	float2 MeshUV : TEXCOORD0,
	float2 ToonUV : TEXCOORD1
) : COLOR {
	return
		tex2D(MeshSmp, MeshUV)*
		tex2D(ToonSmp, ToonUV)*
		Diffuse;
}

//==============================================================
// eNjbN
technique Toon {
	pass P0 {
		CullMode=CCW;
		VertexShader=compile vs_1_1 ToonPaintVS();
		PixelShader=compile ps_1_1 ToonPaintPS();
	}
}

//****************************************************************************
//	gD[CNt
//****************************************************************************
//==============================================================
// _VF[_
void ToonInkVS (
	float4 pos : POSITION,
	float3 normal : NORMAL,
	out float4 Pos : POSITION
) {
	float4x4 wvp = mul(mul(World, View), Proj);
	float4 wvpPos    = mul(pos, wvp);
	float4 wvpNormal = float4(mul(normal, (float3x3)wvp), 0);
	Pos = wvpPos + normalize(wvpNormal) * 0.05f;
}

//==============================================================
// sNZVF[_
float4 ToonInkPS() : COLOR {
	return float4(0, 0, 0, 1);
}

//==============================================================
// eNjbN
technique ToonInk {
	pass P0 {
		CullMode=CCW;
		VertexShader=compile vs_1_1 ToonPaintVS();
		PixelShader=compile ps_1_1 ToonPaintPS();
	}

	pass P1 {
		CullMode=CW;
		VertexShader=compile vs_1_1 ToonInkVS();
		PixelShader=compile ps_1_1 ToonInkPS();
	}
}

//****************************************************************************
//	YIQ
//****************************************************************************
//==============================================================
// sNZVF[_
float4 YIQPS(
	float4 Col : COLOR0,
	float2 MeshUV : TEXCOORD0
) : COLOR {

	// eNX`FƃfBt[YF̍
	float4 c=tex2D(MeshSmpYIQ, MeshUV)*Col;

	// Y
	float3 rgb_to_y={0.299,  0.587,  0.114};
	float y=dot(rgb_to_y, c);

	// RGBA
	float3 i_to_rgb={0.956, -0.272, -1.108};
	return float4(y+i_to_rgb*I, c.a);
}

// eNjbN
technique YIQ {
	pass P0 {
		CullMode=CCW;
		VertexShader=compile vs_1_1 BasicVS();
		PixelShader=compile ps_1_1 YIQPS();
	}
}

//****************************************************************************
//	eNX`̂̍
//****************************************************************************
//==============================================================
// _VF[_
void TexBlendVS (
	float4 pos : POSITION,
	float3 normal : NORMAL,
	float2 mesh_uv : TEXCOORD0,
	out float4 Pos : POSITION,
	out float4 Col : COLOR0,
	out float2 MeshUV : TEXCOORD0,
	out float2 BlendUV : TEXCOORD1
) {
	// _WAeNX`W
	Pos=mul(pos, mul(mul(World, View), Proj));
	MeshUV=mesh_uv;
	BlendUV=mesh_uv;

	// _F
	float3 w_normal=normalize(mul(normal, (float3x3)World));
	Col=max(0, dot(w_normal, -Light))+Ambient;
}

//==============================================================
// sNZVF[_
float4 TexBlendPS(
	float2 MeshUV : TEXCOORD0,
	float2 BlendUV : TEXCOORD1
) : COLOR {
	float4 c0 = tex2D(MeshSmp, MeshUV);
	float4 c1 = tex2D(BlendSmp, BlendUV);

	return c0 + (c1 - c0)*BlendRate;
}

//==============================================================
// eNjbN
technique TexBlend {
	pass P0 {
		CullMode=CCW;
		VertexShader=compile vs_1_1 TexBlendVS();
		PixelShader=compile ps_1_1 TexBlendPS();
	}
}